function init_texteditor()
{
	var icon_el=add_dock_icon(".desktop/Applications/Texteditor/texteditor.png",
	"Text editor","Drop files here to edit them. Click to open a new text editor window.",3,function() {
		open_texteditor();
	});

	make_drop_target(icon_el,
		function(target,files,e) {
			for(var i=0;i<files.length;i++) if(!files[i].dir) return("Edit");
		},
		function(target,files,e) {
			for(var i=0;i<files.length;i++) if(!files[i].dir) open_texteditor(files[i]);
		}
	);

	register_filetype_handler(
		function(file) { return(true) },
		function(file) { open_texteditor(file) },
		-1
	);
}

function open_texteditor(file)
{
	var window_el;

	if(file)
	if(window_el=document.getElementById("texteditor_"+file.fullpath))
	{
		focus_window(window_el);
		return;
	}

	window_el=open_window("Text editor",null,null,600,600);

	window_el.close_handler=function(window_el) {
		close_window(window_el);
	}

	window_el.resize_handler=function(window_el) {
		resize_texteditor(window_el);
	}

	window_el.collapse_handler=function(window_el,collapsed) {
		collapse_texteditor(window_el,collapsed);
	}

	var editor_el=make_element({
		tag: "textarea", wrap: "off",
		onkeypress: function(e) { texteditor_keypress(e,window_el);  },
		onkeydown: function(e) { update_position(window_el) },
		onkeyup: function(e) { update_position(window_el) },
		onmousedown: function(e) { update_position(window_el) },
		onmouseup: function(e) { update_position(window_el) },
		onfocus: function(e) { update_position(window_el) }
	});

	var toolbar_el=make_element({
		tag: "div", className: "toolbar", children: [
			{
				tag: "input",type: "button", value: "Save",
				onclick: function() { save_texteditor_file(window_el,window_el.file) }
			},{
				tag: "input",type: "button", value: "Save and close",
				onclick: function() { save_texteditor_file(window_el,window_el.file,true) }
			},{
				tag: "input",type: "button", value: "Revert",
				onclick: function() {
					set_window_status(window_el,"Reverting to last saved version...");
					load_texteditor_file(window_el,window_el.file);
				}
			},{
				tag: "input",type: "button", value: "Change charset",
				onclick: function() {
					var charset;
					if(charset=prompt("Enter the name of the charset to use for loading:",window_el.charset||""))
					{
						window_el.charset=charset;
						set_window_status(window_el,"Reloading with new charset...");
						load_texteditor_file(window_el,window_el.file);
					}
				}
			},{
				tag: "input",type: "button", value: "Quit",
				onclick: function() { close_window(window_el); }
			}
		]
	});

	var base_el=make_element({
		tag: "div", className: "texteditor",children: [ editor_el,toolbar_el ]
	});
	window_el.content.appendChild(base_el);

	window_el.editor_el=editor_el;
	window_el.toolbar_el=toolbar_el;
	window_el.base_el=base_el;

	make_drop_target(window_el.content,
		function(target,files,e) { if(files.length==1) return("Edit"); else return null; },
		function(target,files,e) { load_texteditor_file(find_containing_window(target),files[0]); }
	);

	init_position(window_el);

	resize_texteditor(window_el);
	if(file) load_texteditor_file(window_el,file);
}

function resize_texteditor(window_el)
{
	set_full_size(window_el.editor_el,
	get_object_w(window_el.content),
	get_object_h(window_el.content)-get_full_h(window_el.toolbar_el));
}

function collapse_texteditor(window_el,collapsed)
{
	var editor_el=window_el.editor_el;
	if(collapsed)
	{
		window_el.stored_scroll_left=editor_el.scrollLeft;
		window_el.stored_scroll_top=editor_el.scrollTop;
		if(editor_el.setSelectionRange)
		{
			window_el.stored_sel_start=editor_el.selectionStart;
			window_el.stored_sel_end=editor_el.selectionEnd;
		}
	}
	else
	{
		editor_el.scrollLeft=window_el.stored_scroll_left;
		editor_el.scrollTop=window_el.stored_scroll_top;
		if(editor_el.setSelectionRange)
		{
			editor_el.setSelectionRange(window_el.stored_sel_start,window_el.stored_sel_end);
		}
	}
}

function texteditor_keypress(e,window_el)
{
	if(!e) e=window.event;

	if(e.keyCode==9)
	{
		texteditor_insert_text(window_el,"\t");
		if(e.preventDefault) e.preventDefault();
	}
	else if(e.charCode==115&&e.ctrlKey) // ctrl-S
	{
		save_texteditor_file(window_el,window_el.file)
		if(e.preventDefault) e.preventDefault();
	}
}

function texteditor_insert_text(window_el,text)
{
	var editor_el=window_el.editor_el;
	if(editor_el.setSelectionRange)
	{
		var left=editor_el.scrollLeft;
		var top=editor_el.scrollTop;
		var start=editor_el.selectionStart;
		var end=editor_el.selectionEnd;
		editor_el.value=editor_el.value.substr(0,start)+text+editor_el.value.substr(end);
		editor_el.setSelectionRange(start+text.length,start+text.length);
		editor_el.scrollLeft=left;
		editor_el.scrollTop=top;
	}
}

function init_position(window_el)
{
	if(document.selection)
	{
/*		editor_el.value="";
		editor_el.select();
		var pos=document.selection.createRange();
		window_el.top_offs=pos.boundingTop;
		window_el.left_offs=pos.boundingLeft;

		editor_el.value=" ";
		editor_el.select();
		var pos=document.selection.createRange();
		window_el.char_width=pos.boundingWidth;
		window_el.char_height=pos.boundingHeight;
		editor_el.value="";*/
	}

	set_window_status(window_el,"");
	update_position(window_el);
}

function update_position(window_el)
{
	var editor_el=window_el.editor_el;

	if(typeof editor_el.selectionStart!="undefined")
	{
		if(window_el.timer_id) clearTimeout(window_el.timer_id);
		window_el.timer_id=setTimeout(function() {
			_update_moz_position(window_el);
			window_el.timer_id=null;
		},get_registry("texteditor.moz.updatedelay",300));
	}
	else if(document.selection)
	{
/*		return;
		var pos=document.selection.createRange();
		var left=(pos.offsetLeft+editor_el.scrollLeft)-window_el.left_offs;
		var top=(pos.offsetTop+editor_el.scrollTop)-window_el.top_offs;

		line=(top/window_el.char_height)+1;
		col=(left/window_el.char_width)+1;

		var status="Line: "+line+", Column: "+col;
		set_window_status(window_el,status);*/
	}
	else set_window_status(window_el,"");
}

function _update_moz_position(window_el)
{
	var editor_el=window_el.editor_el;
	var line,col;
	var sel_lines,sel_chars;

	line=count_newlines(editor_el.value,editor_el.selectionStart)+1;
	col=find_line_start(editor_el.value,editor_el.selectionStart);

	sel_chars=editor_el.selectionEnd-editor_el.selectionStart;
	if(sel_chars) sel_lines=count_newlines(editor_el.value.substr(editor_el.selectionStart,sel_chars))+1;

	var status="Line: "+line+", Column: "+col;
	if(sel_chars) status+=" ("+pluralize(sel_lines,"line")+", "+pluralize(sel_chars,"character")+")";

	set_window_status(window_el,status);
}

function count_newlines(text,pos)
{
	var n=0;
	for(var i=0;i<pos;i++) if(text.charCodeAt(i)==10) n++;
	return(n);
}

function find_line_start(text,pos)
{
	var n=1;
	while(n<=pos&&text.charCodeAt(pos-n)!=10) n++;
	return(n);
}

function load_texteditor_file(window_el,file)
{
	var editor_el=window_el.editor_el;

	set_window_status(window_el,"Loading file...");
	if(window_el.charset) file.charset=window_el.charset;

	file.read_content(function(file,error) {
		if(error)
		{
			set_window_status(window_el,"Couldn't load file: "+error);
		}
		else
		{
			set_texteditor_file(window_el,file);

			editor_el.value=file.content;

			editor_el.focus();
			if(editor_el.setSelectionRange) editor_el.setSelectionRange(0,0);
			// else do IE things

			update_position(window_el);
		}
	});
}

function save_texteditor_file(window_el,file,close)
{
	var editor_el=window_el.editor_el;

	set_window_status(window_el,"Saving file...");

	file.content=editor_el.value;

	file.write_content(function(file,error) {
		if(error)
		{
			set_window_status(window_el,"Saving the file failed: "+error);
		}
		else
		{
			set_texteditor_file(window_el,file);
			if(close) close_window(window_el);
			else set_window_status(window_el,"File saved successfully!");
		}
	});
}

function set_texteditor_file(window_el,file)
{
	window_el.file=file;
	window_el.id="texteditor_"+file.fullpath;

	var title=file.filename;
	if(file.charset) title+" ("+file.charset+")";
	set_window_title(window_el,title);
}

register_application(init_texteditor);
