use strict;

sub make_dirlist($)
{
	my $query=shift;
	my $path=new fileobj($query->param("path"));
	my @files=$path->readdir();

	return_result(files=>js_array(map { $_->get_data() } @files));
}

sub read_file($)
{
	my $query=shift;
	my $file=new fileobj($query->param("file"));
	my $charset=$query->param("charset");
	my $buf;

	open FILE,$file->realpath() or return_result(error=>js_string($!));

	my $content;
	binmode FILE;
	$content.=$_ while(<FILE>);
	close FILE;

	$charset=guess_charset($content) unless($charset);
	$content=decode_string($content,$charset);

	return_result(content=>js_string($content),charset=>js_string($charset));
}

sub write_file($)
{
	my $query=shift;
	my $file=new fileobj($query->param("file"));
	my $content=$query->param("content");
	my $charset=$query->param("charset");

	$charset="utf-8" unless($charset);

	$content=encode_string(decode_string($content,"utf8"),$charset);

	open FILE,">".$file->realpath() or return_result(error=>js_string($!));
	binmode FILE;
	print FILE $content; # should check for error
	close FILE;

	return_result(newfile=>$file->get_data());
}

sub rename_file($)
{
	my $query=shift;
	my $file=new fileobj($query->param("file"));
	my $newname=$query->param("newname");
	my $newfile=new fileobj($file->directory(),$newname);

	if(rename $file->realpath(),$newfile->realpath()) { return_result(newfile=>$newfile->get_data()) }
	else { return_result(error=>js_string($!)) }
}

sub chmod_files($)
{
	my $query=shift;
	my @files=map { new fileobj($_) } $query->param("file");
	my $permissions=$query->param("permissions");
	my (@success,@failure);

	for my $file (@files)
	{
		if(chmod oct($permissions),$file->realpath()) { push @success,$file->get_data() }
		else { push @failure,js_string($file->fullpath()) };
	}

	return_result(success=>js_array(@success),failure=>js_array(@failure));
}

sub delete_files($)
{
	my $query=shift;
	my @files=map { new fileobj($_) } $query->param("file");
	my @failure;

	for my $file (@files)
	{
		unless(delete_file($file)) { push @failure,js_string($file->fullpath()) }
	}

	return_result(failure=>js_array(@failure));
}

sub delete_file($)
{
	my ($file)=@_;

	if($file->is_directory())
	{
		my @files=$file->readdir();

		foreach my $subfile (@files) { delete_file($subfile) }

		rmdir $file->realpath() or return 0;
	}
	else
	{
		unlink $file->realpath() or return 0;
	}
	return 1;
}

sub make_directory($)
{
	my $query=shift;
	my $path=new fileobj($query->param("path"));

	mkdir $path->realpath() or return_result(error=>js_string($!));

	return_result(newfile=>$path->get_data());
}

our %task_handlers;
$task_handlers{dirlist}=\&make_dirlist;
$task_handlers{readfile}=\&read_file;
$task_handlers{writefile}=\&write_file;
$task_handlers{'rename'}=\&rename_file;
$task_handlers{'chmod'}=\&chmod_files;
$task_handlers{'delete'}=\&delete_files;
$task_handlers{'makedir'}=\&make_directory;

1;
