use strict;

use File::Copy;

sub move_files($)
{
	my $query=shift;
	my @files=map { new fileobj($_) } $query->param("file");
	my $path=new fileobj($query->param("path"));
	my (@success,@failure);

	for my $file (@files)
	{
		my $newfile=new fileobj($path,$file->filename());

		next if($newfile->fullpath() eq $file->fullpath()); # shouldn't happen

		if(move_file($file,$newfile,0)) { push @success,$newfile->get_data() }
		else { push @failure,js_string($file->fullpath()) };
	}

	return_result(success=>js_array(@success),failure=>js_array(@failure));
}

sub copy_files($)
{
	my $query=shift;
	my @files=map { new fileobj($_) } $query->param("file");
	my $path=new fileobj($query->param("path"));
	my (@success,@failure);

	for my $file (@files)
	{
		my $filename=$file->filename();
		my $newfile=new fileobj($path,$filename);

		if($newfile->fullpath() eq $file->fullpath())
		{
			$filename=~s/^Copy (?:[0-9]+ |)of //; # remove previous "Copy of " to avoid repetition.
			$newfile=new fileobj($path,"Copy of $filename");

			my $counter=1;
			while(-e $newfile->realpath())
			{
				$counter++;
				$newfile=new fileobj($path,"Copy $counter of $filename");
			}
		}

		if(move_file($file,$newfile,1)) { push @success,$newfile->get_data() }
		else { push @failure,js_string($file->fullpath()) };
	}

	return_result(success=>js_array(@success),failure=>js_array(@failure));
}

sub move_file($$$)
{
	my ($src,$dest,$copy)=@_;
	my $srcpath=$src->realpath();
	my $destpath=$dest->realpath();

	if($src->is_directory())
	{
		my @files=$src->readdir();

		mkdir $destpath or return 0 if(!$dest->is_directory());

		foreach my $srcfile (@files)
		{
			my $destfile=new fileobj($dest,$srcfile->filename());
			move_file($srcfile,$destfile,$copy) or return 0;
		}

		rmdir $srcpath or return 0 unless($copy);
	}
	#elsif($src->is_link())
	#{
	#	symlink readlink($srcpath),$destpath or return 0;
	#}
	else
	{
		if($copy) { copy($srcpath,$destpath) or return 0 }
		else { move($srcpath,$destpath) or return 0 }
		#chmod scalar((stat $srcpath)[2]),$destpath;
	}
	return 1;
}

our %task_handlers;
$task_handlers{move}=\&move_files;
$task_handlers{copy}=\&copy_files;


1;
