use strict;

our %config;

sub decode_string($$)
{
	my ($string,$charset)=@_;
	if($]>5.007)
	{
		require Encode;

		if(Encode::is_utf8($string)) { return $string }
		else { return Encode::decode($charset,$string) }
	}
	else
	{
		return $string; # do some decoding by hand
	}
}

sub encode_string($$)
{
	my ($string,$charset)=@_;
	if($]>5.007)
	{
		require Encode;

		return Encode::encode($charset,$string);
	}
	else
	{
		return $string;
	}
}

sub guess_charset($)
{
	my $string=shift;
	return "utf8";
}



package fileobj;

use File::Spec;
use Fcntl ':mode';

sub new
{
	my $invocant=shift;
	my $class=ref $invocant||$invocant;
	my $self=bless [],$class;

	$self->construct(@_);

	return $self;
}

sub construct($@)
{
	my $self=shift;
	my @components=grep $_ ne "",map {
		if(ref $_ eq "fileobj") { $_->components() }
		else { split /\//,::decode_string($_,"utf8") }
	} @_;
	@$self=@components;
}

sub components($)
{
	return @{(shift)};
}

sub realpath($)
{
	my @self=@{(shift)};
	return ::encode_string(
		File::Spec->rel2abs(File::Spec->catfile(File::Spec->no_upwards(@self)),$::config{base_path}),
		$::config{fs_charset});
}

sub filename($)
{
	my @self=@{(shift)};
	return $self[$#self];
}

sub directory($)
{
	my @self=@{(shift)};
	return "/".join "/",@self[0..$#self-1];
}

sub directory_obj($)
{
	my @self=@{(shift)};
	return new fileobj(@self[0..$#self-1]);
}

sub fullpath($)
{
	my @self=@{(shift)};
	return "/".join "/",@self;
}



sub exists($)
{
	my $self=shift;
	return -e $self->realpath();
}

sub is_directory($)
{
	my $self=shift;
	return -d $self->realpath();
}

sub is_link($)
{
	my $self=shift;
	return -l $self->realpath();
}



sub readdir($)
{
	my $self=shift;

	opendir my $dir,$self->realpath() or return 0;
	my @files=grep { $_ ne "." and $_ ne ".." } readdir $dir;
	closedir $dir;

	return map { new fileobj($self,::decode_string($_,$::config{fs_charset})) } @files;
}



sub get_data($)
{
	my $self=shift;

	die $self->fullpath() unless -e $self->realpath();

	my @stat=stat $self->realpath();
	my %file;

	$file{filename}=::js_string($self->filename());
	$file{size}=$stat[7];
	$file{modify}=$stat[9];
	$file{permissions}=::js_string(sprintf("%03o",S_IMODE($stat[2])));
	$file{dir}=1 if(S_ISDIR($stat[2]));
	#$file{link}=1 if(S_ISLNK($stat[2]));

	my @pw;
	eval "@pw=getpwuid $stat[4]";
	$file{owner}=::js_string($pw[0]) unless($@);

	my @gr;
	eval "@gr=getgrgid $stat[5]";
	$file{group}=::js_string($gr[0]) unless($@);

	return ::js_hash(%file);
}



1;
